/***************************************************************************
                          guarddog.cpp  -  description
                             -------------------
    begin                : Thu Feb 10 20:57:36 EST 2000
    copyright            : (C) 2000 by Simon Edwards
    email                : simon@simonzone.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qdir.h>
#include <qstrlist.h>
#include <qprinter.h>
#include <qpainter.h>

// include files for KDE
#include <kiconloader.h>
#include <kmsgbox.h>
#include <kfiledialog.h>
#include <unistd.h>
#include <sys/stat.h>

// application specific includes
#include "guarddog.h"
#include "guarddogview.h"
#include "guarddogdoc.h"
#include "resource.h"
#include "protocoldb.h"

///////////////////////////////////////////////////////////////////////////
GuardDogApp::GuardDogApp() {
#define ERRORMSG_BUFFER_SIZE    2048
  	char errormsg[ERRORMSG_BUFFER_SIZE];
    QString *protocollocation = 0;

	config=kapp->getConfig();
		// Set the root user mode flag.
	sumode = getuid()==0;
	
	///////////////////////////////////////////////////////////////////
	// call inits to invoke all other construction parts
	initMenuBar();
	initToolBar();
	initStatusBar();
	initKeyAccel();

		// Read in the Protocol Database.
	pdb = new ProtocolDB();
    protocollocation = new QString((const char *)(kapp->kde_datadir()));
	protocollocation->append("/guarddog/protocoldatabase");
	if(!pdb->read((const char *)(*protocollocation))) {
    	snprintf(errormsg,ERRORMSG_BUFFER_SIZE,"An error occured while reading the protocol database.\n\nDetails: \"%s\"",pdb->lastError());
    	KMsgBox::message(0,i18n("GuardDog"),errormsg);
	}
	
	initDocument();
	initView();
	
	readOptions();

  ///////////////////////////////////////////////////////////////////
  // disable menu and toolbar items at startup
//  disableCommand(ID_FILE_SAVE);
//  disableCommand(ID_FILE_SAVE_AS);
//  disableCommand(ID_FILE_PRINT);
	if(sumode) {
		slotStatusMsg(i18n("Running in super user mode. Ready"));
	} else {
		slotStatusMsg(i18n("Running is user mode. Ok and Apply commands are not available. Ready"));
	}
	delete protocollocation;
}

///////////////////////////////////////////////////////////////////////////
GuardDogApp::~GuardDogApp()
{
    delete pdb;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::initKeyAccel()
{
  keyAccel = new KAccel(this);
	
  // fileMenu accelerators
  keyAccel->connectItem(KAccel::New, this, SLOT(slotFileNew()));
  keyAccel->connectItem(KAccel::Open, this, SLOT(slotFileOpen()));
  keyAccel->connectItem(KAccel::Save, this, SLOT(slotFileSave()));
  keyAccel->connectItem(KAccel::Close, this, SLOT(slotFileClose()));
  keyAccel->connectItem(KAccel::Quit, this, SLOT(slotFileQuit()));

  keyAccel->connectItem(KAccel::Help, kapp, SLOT(appHelpActivated()));
			
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_NEW, KAccel::New);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_OPEN, KAccel::Open);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_SAVE, KAccel::Save);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_CLOSE, KAccel::Close);
  keyAccel->changeMenuAccel(fileMenu, ID_FILE_QUIT, KAccel::Quit);

  keyAccel->readSettings();	
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::initMenuBar()
{
	if(sumode) {
		menuBar()->setBackgroundColor(red);
	}
	
  ///////////////////////////////////////////////////////////////////
  // MENUBAR
  recentFilesMenu = new QPopupMenu();
  connect(recentFilesMenu, SIGNAL(activated(int)), SLOT(slotFileOpenRecent(int)));

  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu
  fileMenu = new QPopupMenu();
  fileMenu->insertItem(Icon("filenew.xpm"), i18n("&New"), ID_FILE_NEW);
  fileMenu->insertItem(Icon("fileopen.xpm"), i18n("&Open..."), ID_FILE_OPEN);
  fileMenu->insertItem(i18n("Open &recent"), recentFilesMenu, ID_FILE_OPEN_RECENT);

  fileMenu->insertSeparator();
  fileMenu->insertItem(Icon("filefloppy.xpm") ,i18n("&Save"), ID_FILE_SAVE);
  fileMenu->insertItem(i18n("Save &As..."), ID_FILE_SAVE_AS);
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("E&xit"), ID_FILE_QUIT);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  viewMenu = new QPopupMenu();
  viewMenu->setCheckable(true);
  viewMenu->insertItem(i18n("&Toolbar"), ID_VIEW_TOOLBAR);
  viewMenu->insertItem(i18n("&Statusbar"), ID_VIEW_STATUSBAR);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  QString aboutstring=kapp->appName()+" "+VERSION+"\n" +
  	"by Simon Edwards <simon@simonzone.com>\n" +
  	"\nFirewall configuration utility.\n";

  helpMenu = new QPopupMenu();
  helpMenu = kapp->getHelpMenu(true, aboutstring);

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  // insert your popup menus with the according menubar entries in the order
  // they will appear later from left to right
  menuBar()->insertItem(i18n("&File"), fileMenu);
  menuBar()->insertItem(i18n("&View"), viewMenu);

  menuBar()->insertSeparator();
  menuBar()->insertItem(i18n("&Help"), helpMenu);

  ///////////////////////////////////////////////////////////////////
  // CONNECT THE MENU SLOTS WITH SIGNALS
  // for execution slots and statusbar messages

  connect(fileMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  connect(fileMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));

  connect(viewMenu, SIGNAL(activated(int)), SLOT(commandCallback(int)));
  connect(viewMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));

}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::initToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  toolBar()->insertButton(Icon("filenew.xpm"), ID_FILE_NEW, true, i18n("New File"));
  toolBar()->insertButton(Icon("fileopen.xpm"), ID_FILE_OPEN, true, i18n("Open File"));
  toolBar()->insertButton(Icon("filefloppy.xpm"), ID_FILE_SAVE, true, i18n("Save File"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("help.xpm"), ID_HELP_CONTENTS, SIGNAL(clicked()),
  				kapp, SLOT(appHelpActivated()), true,i18n("Help"));

  ///////////////////////////////////////////////////////////////////
  // INSERT YOUR APPLICATION SPECIFIC TOOLBARS HERE WITH toolBar(n)


  ///////////////////////////////////////////////////////////////////
  // CONNECT THE TOOLBAR SLOTS WITH SIGNALS - add new created toolbars by their according number
  // connect for invoking the slot actions
  connect(toolBar(), SIGNAL(clicked(int)), SLOT(commandCallback(int)));
  // connect for the status help on holing icons pressed with the mouse button
  connect(toolBar(), SIGNAL(pressed(int)), SLOT(statusCallback(int)));

}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::initStatusBar()
{
  statusBar()->insertItem(i18n(IDS_STATUS_DEFAULT), ID_STATUS_MSG);
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::initDocument()
{
  doc = new GuardDogDoc(this);
  doc->newDocument();
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new GuardDogView(this);
  doc->addView(view);
  setView(view);
  QString caption=kapp->getCaption();	
  setCaption(caption+": "+doc->getTitle());

}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::enableCommand(int id_)
{
  ///////////////////////////////////////////////////////////////////
  // enable menu and toolbar functions by their ID's
  menuBar()->setItemEnabled(id_, true);
  toolBar()->setItemEnabled(id_, true);
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::disableCommand(int id_)
{
  ///////////////////////////////////////////////////////////////////
  // disable menu and toolbar functions by their ID's
  menuBar()->setItemEnabled(id_, false);
  toolBar()->setItemEnabled(id_, false);
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::addRecentFile(const QString &file)
{
  if(recentFiles.find(file) == -1)
  {
    if( recentFiles.count() < 5)
    {
      recentFiles.insert(0, file);
    }
    else
    {
      recentFiles.remove(4);
      recentFiles.insert(0, file);
    }
    recentFilesMenu->clear();
    for ( int i=0 ; i < (int) recentFiles.count(); i++)
    {
      recentFilesMenu->insertItem(recentFiles.at(i));
    }
  }
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::openDocumentFile(const char * _cmdl)
{
	QString errorstring;
	int rc;
	
	slotStatusMsg(i18n("Opening file..."));

	rc = doc->openDocument(_cmdl,errorstring);
	if(rc!=GuardDogDoc::OK) {
		KMsgBox::message(this,i18n("GuardDog"),errorstring);
	}
	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::openDefault() {
	int rc;
	QString caption;
	QString *filetoopen = new QString(SYSTEM_RC_FIREWALL);
	
	rc = doc->openDefault(SYSTEM_RC_FIREWALL);
	if(rc==GuardDogDoc::OK) {
 		addRecentFile(*filetoopen);
   		slotStatusMsg(i18n("Firewall opened."));
	}
    caption = kapp->getCaption();
   	setCaption(caption+": "+doc->getTitle());
	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

///////////////////////////////////////////////////////////////////////////
GuardDogDoc *GuardDogApp::getDocument() const
{
  return doc;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::saveOptions()
{	
  config->setGroup("General Options");
  config->writeEntry("Geometry", size());
  config->writeEntry("Show Toolbar", toolBar()->isVisible());
  config->writeEntry("Show Statusbar",statusBar()->isVisible());
  config->writeEntry("MenuBarPos", (int) menuBar()->menuBarPos());
  config->writeEntry("ToolBarPos", (int) toolBar()->barPos());
  config->writeEntry("Recent Files", recentFiles);
}


///////////////////////////////////////////////////////////////////////////
void GuardDogApp::readOptions()
{
	
  config->setGroup("General Options");

  // bar status settings
  bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
  viewMenu->setItemChecked(ID_VIEW_TOOLBAR, bViewToolbar);
  if(!bViewToolbar)
  {
     enableToolBar(KToolBar::Hide);
  }
	
  bool bViewStatusbar = config->readBoolEntry("Show Statusbar", true);
  viewMenu->setItemChecked(ID_VIEW_STATUSBAR, bViewStatusbar);
  if(!bViewStatusbar)
  {
    enableStatusBar(KStatusBar::Hide);
  }

  // bar position settings
  KMenuBar::menuPosition menuBarPos;
  menuBarPos=(KMenuBar::menuPosition) config->readNumEntry("MenuBarPos", KMenuBar::Top);

  KToolBar::BarPosition toolBarPos;
  toolBarPos=(KToolBar::BarPosition) config->readNumEntry("ToolBarPos", KToolBar::Top);

  menuBar()->setMenuBarPos(menuBarPos);
  toolBar()->setBarPos(toolBarPos);
	
  // initialize the recent file list
  recentFiles.setAutoDelete(TRUE);
  config->readListEntry("Recent Files", recentFiles);
	
  for (int i=0; i < (int) recentFiles.count(); i++)
  {
    recentFilesMenu->insertItem(recentFiles.at(i));
  }

  QSize size=config->readSizeEntry("Geometry");
  if(!size.isEmpty())
  {
    resize(size);
  }
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::saveProperties(KConfig *_cfg)
{
	QString errorstring;
	int rc;
	
	if(!doc->isFilePathValid() && !doc->isModified()) {
	    // saving to tempfile not necessary
	} else {
    	QString filename = doc->getAbsFilePath();	
    	_cfg->writeEntry("filename", filename);
    	_cfg->writeEntry("modified", doc->isModified());
		
    	QString tempname = kapp->tempSaveName(filename);
    	
    	rc = doc->saveDocument(tempname,errorstring);
			// Unfortunately we can't do error checking and then
			// talk to the user....
  	}
}


///////////////////////////////////////////////////////////////////////////
void GuardDogApp::readProperties(KConfig* _cfg)
{
	QString filename = _cfg->readEntry("filename", "");
	bool modified = _cfg->readBoolEntry("modified", false);
	QString errorstring;
	
  	if(modified) {
    	bool canRecover;
    	QString tempname = kapp->checkRecoverFile(filename, canRecover);
  	
    	if(canRecover) {
      		doc->openDocument(tempname,errorstring);
      		doc->setModified();
      		QFileInfo info(filename);
      		doc->setAbsFilePath(info.absFilePath());
      		QFile::remove(tempname);
    	}
  	} else {
    	if(!filename.isEmpty()) {
      		doc->openDocument(filename,errorstring);
    	}
  	}

  	QString caption = kapp->getCaption();	
  	setCaption(caption + ": " + doc->getTitle());
}		

///////////////////////////////////////////////////////////////////////////
bool GuardDogApp::queryClose()
{
  return true;
}

///////////////////////////////////////////////////////////////////////////
bool GuardDogApp::queryExit()
{
  saveOptions();
  return true;
}


///////////////////////////////////////////////////////////////////////////
bool GuardDogApp::saveModified() {
    int want_save;

	if(doc->isModified()) {
    	want_save = KMsgBox::yesNoCancel(this, i18n("Warning"),
        	i18n("The current file has been modified.\n"
        	"Do you want to save it?"));
    	switch(want_save) {
      		case 1:
            	if(slotFileSave()) {
            		doc->deleteContents();
            		return true;
            	} else {
            		return false;
            	}

      		case 2:
           		doc->setModified(false);
           		doc->deleteContents();
           		return true;

      		case 3:
      			return false;

      		default:
           		return false;
    	}
	}
	return true;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::slotFileNew()
{
	QString caption;
	slotStatusMsg(i18n("Creating new firewall..."));

	if(saveModified()) {
    	doc->newDocument();		
		caption = kapp->getCaption();	
    	setCaption(caption+": "+doc->getTitle());
  	}

  	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::slotFileOpen()
{
	QString errorstring;
	QString fileToOpen;
	QString caption;
	int rc;
	
	slotStatusMsg(i18n("Opening file..."));
	if(saveModified()) {	
    	fileToOpen = KFileDialog::getOpenFileName(QDir::homeDirPath(),
    		i18n("*|All files"), this, i18n("Open File..."));
    	if(!fileToOpen.isEmpty()) {
      		rc = doc->openDocument(fileToOpen,errorstring);
      		if(rc==GuardDogDoc::OK) {
	      		caption = kapp->getCaption();
    	  		setCaption(caption+": "+doc->getTitle());
	      		addRecentFile(fileToOpen);
	      		slotStatusMsg(i18n("Firewall opened."));
	      		return;
      		} else {
      				// Something went wrong, error message time.
				KMsgBox::message(this,i18n("GuardDog"),errorstring);
      			slotStatusMsg(i18n("Error opening firewall."));
      			return;
      		}
    	}
	}

	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::slotFileOpenRecent(int id_)
{
	QString errorstring;
	QString caption;
	int rc;
	
  	slotStatusMsg(i18n("Opening file..."));
	
  	if(saveModified()) {
    	rc = doc->openDocument(recentFiles.at(id_),errorstring);
    	if(rc==GuardDogDoc::OK) {
    		caption = kapp->getCaption();	
    		setCaption(caption+": "+doc->getTitle());
		  	slotStatusMsg(i18n("Firewall opened."));
			return;    		
    	} else {
			KMsgBox::message(this,i18n("GuardDog"),errorstring);
    	}
  	}
  	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

///////////////////////////////////////////////////////////////////////////
bool GuardDogApp::slotFileSave() {
	QString errorstring;
	int rc;
	bool allcool;
	allcool = true;
	
	slotStatusMsg(i18n("Saving file..."));
	
	if(doc->isFilePathValid()) {
		rc = doc->saveDocument(doc->getAbsFilePath(),errorstring);
		if(rc!=GuardDogDoc::OK) {
				// If things are not cool, then we stick up an error message.
			KMsgBox::message(this,i18n("GuardDog"),errorstring);
			allcool = false;
			slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
		} else {
			slotStatusMsg(i18n("Firewall saved."));
		}
	} else {
		allcool = slotFileSaveAs();
	}
	return allcool;
}

///////////////////////////////////////////////////////////////////////////
bool GuardDogApp::slotFileSaveAs() {
	QString errorstring;
	QString newName;
	QString caption;
	QString askstring;
	int rc;
	bool allcool;
	struct stat statbuffer;
	
	allcool = true;
	slotStatusMsg(i18n("Saving file with a new filename..."));

	newName = KFileDialog::getSaveFileName(QDir::currentDirPath(),
    	i18n("*|All files"), this, i18n("Save as..."));
  	if(!newName.isEmpty()) {
			// Does this file exist?
		if(stat((const char *)newName,&statbuffer)>=0) {
				// -----------------------------------------------
				// The file already exists.
				// We will ask if they really wanted to over write that file.
			askstring = "Do you wish to overwrite the file '";
			askstring += (const char *)newName;
			askstring += "'?";
			if(KMsgBox::yesNo(this, i18n("GuardDog"),askstring,
					KMsgBox::EXCLAMATION, i18n("Ok"), i18n("Cancel"))==false) {
				slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
				return false;
			}
    	}

//    		// Are we about to overwrite the systems current firewall?
//			// Warn them that they are about to modify the system's firewall config.
//		if(strcmp(SYSTEM_RC_FIREWALL,(const char *)newName)==0) {
//      	    if(KMsgBox::yesNo(this, i18n("GuardDog"),
//           			i18n("Are about to modify the system's firewall configuration in '"
//            		SYSTEM_RC_FIREWALL
//   	        		"'.\n"
//       	    		"These changes will not take effect immediately. They will take effect\n"
//           			" the next time the system reboots.\n"),
//            		KMsgBox::EXCLAMATION, i18n("Ok"), i18n("Cancel"))==false) {
//   	        	return false;
//       	    }
//		}

    	rc = doc->saveDocument(newName,errorstring);
    	if(rc==GuardDogDoc::OK) {
    			// Saved just fine.
		    QFileInfo saveAsInfo(newName);
		    doc->setAbsFilePath(saveAsInfo.absFilePath());
		    addRecentFile(newName);
	    	caption = kapp->getCaption();
		    setCaption(caption+": "+doc->getTitle());
		    slotStatusMsg(i18n("Firewall saved."));
		    return true;
    	} else {
    			// There was a problem saving the file.
			KMsgBox::message(this,i18n("GuardDog"),errorstring);
    		slotStatusMsg(i18n("Error saving firewall."));
    		return false;
    	}
	} else {
		slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
		return false;
	}
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::slotFileClose()
{
  slotStatusMsg(i18n("Closing file..."));
	
  close();
  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::slotFileQuit() {
	slotStatusMsg(i18n("Exiting..."));
	saveOptions();
		// close the first window, the list makes the next one the first again.
		// This ensures that queryClose() is called on each window to ask for closing
	KTMainWindow* w;
	if(memberList) {
		for(w=memberList->first(); w!=0; w=memberList->first()) {
      		// only close the window if the closeEvent is accepted. If the user presses
      		// Cancel on the saveModified() dialog, the window and the application stay open.
 			if(!w->close())
				break;
		}
	}	
	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::slotOk() {
	QString errorstring;
	int rc;
	
	if(doc->isFilePathValid()) {
		rc = doc->saveDocument(doc->getAbsFilePath(),errorstring);
		if(rc!=GuardDogDoc::OK) {
			KMsgBox::message(this,i18n("GuardDog"),errorstring);
			return;
		}
	} else {
		if(slotFileSaveAs()==false) {
			return;
		}
	}
	if(slotApply()) {
		slotFileQuit();
	}
}

///////////////////////////////////////////////////////////////////////////
bool GuardDogApp::slotApply() {
	QString errorstring;
	int rc;
	bool allcool;
	allcool = true;
	
	slotStatusMsg(i18n("Installing Firewall."));
	
	if(KMsgBox::yesNo(this, i18n("GuardDog"),
  			i18n("You are about to modify the system's firewall configuration.\n"
            "These changes may disrupt current network connections.\n\n"
            "Do you wish to continue?"),
            KMsgBox::EXCLAMATION, i18n("Ok"), i18n("Cancel"))) {
		
		rc = doc->applyFirewall(errorstring);
		if(rc!=GuardDogDoc::OK) {
			KMsgBox::message(this,i18n("GuardDog"),errorstring);
			allcool = false;
		} else {
		 	slotStatusMsg(i18n("Firewall installed."));
		 	return true;
		}
    }
 	slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
 	return allcool;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::slotViewToolBar()
{
  slotStatusMsg(i18n("Toggle the toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  if( viewMenu->isItemChecked(ID_VIEW_TOOLBAR))
  {
    viewMenu->setItemChecked(ID_VIEW_TOOLBAR, false);
    enableToolBar(KToolBar::Hide);
  }
  else
  {
    viewMenu->setItemChecked(ID_VIEW_TOOLBAR, true);
    enableToolBar(KToolBar::Show);
  }		

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  if( viewMenu->isItemChecked(ID_VIEW_STATUSBAR))
  {
    viewMenu->setItemChecked(ID_VIEW_STATUSBAR, false);
    enableStatusBar(KStatusBar::Hide);
  }
  else
  {
    viewMenu->setItemChecked(ID_VIEW_STATUSBAR, true);
    enableStatusBar(KStatusBar::Show);
  }

  slotStatusMsg(i18n(IDS_STATUS_DEFAULT));
}

///////////////////////////////////////////////////////////////////////////

void GuardDogApp::slotStatusMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}


///////////////////////////////////////////////////////////////////////////
void GuardDogApp::slotStatusHelpMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 2000);
}



///////////////////////////////////////////////////////////////////////////
void GuardDogApp::commandCallback(int id_)
{
  switch (id_)
  {
    case ID_FILE_NEW:
    	 slotFileNew();
         break;

    case ID_FILE_OPEN:
         slotFileOpen();
         break;

    case ID_FILE_SAVE:
         slotFileSave();
         break;

    case ID_FILE_SAVE_AS:
         slotFileSaveAs();
         break;

    case ID_FILE_CLOSE:
         slotFileClose();
         break;

    case ID_FILE_QUIT:
         slotFileQuit();
         break;

    case ID_VIEW_TOOLBAR:
         slotViewToolBar();
         break;

    case ID_VIEW_STATUSBAR:
         slotViewStatusBar();
         break;

    default:
         break;
  }
}

///////////////////////////////////////////////////////////////////////////
void GuardDogApp::statusCallback(int id_)
{
  switch (id_)
  {
    case ID_FILE_NEW:
         slotStatusHelpMsg(i18n("Creates a new document"));
         break;

    case ID_FILE_OPEN:
         slotStatusHelpMsg(i18n("Opens an existing document"));
         break;

    case ID_FILE_OPEN_RECENT:
         slotStatusHelpMsg(i18n("Opens a recently used file"));
         break;

    case ID_FILE_SAVE:
         slotStatusHelpMsg(i18n("Saves the actual document"));
         break;

    case ID_FILE_SAVE_AS:
         slotStatusHelpMsg(i18n("Saves the actual document as..."));
         break;

    case ID_FILE_CLOSE:
         slotStatusHelpMsg(i18n("Closes the actual document"));
         break;

    case ID_FILE_QUIT:
         slotStatusHelpMsg(i18n("Quits the application"));
         break;

    case ID_VIEW_TOOLBAR:
         slotStatusHelpMsg(i18n("Enables/disables the toolbar"));
         break;

    case ID_VIEW_STATUSBAR:
         slotStatusHelpMsg(i18n("Enables/disables the statusbar"));
         break;

    default:
         break;
  }
}
