/***************************************************************************
                          guarddogdoc.cpp  -  description
                             -------------------
    begin                : Thu Feb 10 20:57:36 EST 2000
    copyright            : (C) 2000 by Simon Edwards
    email                : simon@simonzone.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>

// include files for KDE
#include <kapp.h>
#include <kmsgbox.h>
#include <kprocess.h>
#include <errno.h>
#include <sys/stat.h>
#include <unistd.h>

// application specific includes
#include "guarddogdoc.h"
#include "guarddog.h"
#include "guarddogview.h"
#include "protocoldb.h"

QList<GuardDogView> *GuardDogDoc::pViewList = 0L;

///////////////////////////////////////////////////////////////////////////

GuardDogDoc::GuardDogDoc(QWidget *parent, const char *name) : QObject(parent, name)
{
  	int count;
  	
  	if(!pViewList) {
   		pViewList = new QList<GuardDogView>();
  	}

	pdb = ((GuardDogApp *)parent)->pdb;

	filePathValid = false;
	
	allowedClientProtocols = 0;
	allowedServerProtocols = 0;

		// This list of bools holds the list of what protocols should be turned on/off.
	count = pdb->clientsList.count();
	allowedClientProtocols = new bool[count];
	
	count = pdb->serversList.count();
	allowedServerProtocols = new bool[count];
	
	factoryDefaults();
}

///////////////////////////////////////////////////////////////////////////
GuardDogDoc::~GuardDogDoc()
{
	delete[] allowedClientProtocols;
	delete[] allowedServerProtocols;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::addView(GuardDogView *view)
{
  pViewList->append(view);
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::removeView(GuardDogView *view)
{
  pViewList->remove(view);
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::setAbsFilePath(const QString &filename)
{
  absFilePath=filename;
  filePathValid = true;
}
	
///////////////////////////////////////////////////////////////////////////
bool GuardDogDoc::isFilePathValid() {
	return filePathValid;
}

///////////////////////////////////////////////////////////////////////////
const QString &GuardDogDoc::getAbsFilePath() const
{
  return absFilePath;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::setTitle(const QString &_t)
{
  title=_t;
}

///////////////////////////////////////////////////////////////////////////
const QString &GuardDogDoc::getTitle() const
{
  return title;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::slotUpdateAllViews(GuardDogView *sender)
{
  GuardDogView *w;
  if(pViewList)
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
      if(w!=sender)
        w->repaint();
    }
  }

}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::closeDocument()
{
  deleteContents();
}

///////////////////////////////////////////////////////////////////////////
bool GuardDogDoc::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
	modified = false;
	absFilePath = QDir::homeDirPath();
	title = i18n("Untitled");
	factoryDefaults();
	return true;
}

///////////////////////////////////////////////////////////////////////////
int GuardDogDoc::openDocument(const QString &filename,QString &errorstring)
{
	FILE *fhandle;
	QFileInfo fileInfo(filename);
	int rc;
		
	title = fileInfo.fileName();
	absFilePath = fileInfo.absFilePath();	

	fhandle = fopen((const char *)filename,"r");
	if(fhandle==0) {
		errorstring = "An error occurred while opening '";
		errorstring += filename;
		errorstring += "' for writing. The operating system has this to report about the error: ";
		errorstring += strerror(errno);
		return READ_ERROR;
	}
	
	rc = readFirewall(fhandle,errorstring);
	
	fclose(fhandle);
	slotUpdateAllViews(0);
	modified = false;
	if(rc==OK) {
		filePathValid = true;
		setTitle(filename);
	}
	return rc;
}

///////////////////////////////////////////////////////////////////////////
// *** ATTENTION
// *** If you are here to port this code to another OS. Make sure you
// *** check the linux mktemps(3) man page against whatever you are porting to.
// *** The security of the the tmp file's permission bits may depend on it...
//
int GuardDogDoc::saveDocument(const QString &filename, QString &errorstring)
{
	FILE *fhandle;
	int tmpfd;
	struct stat statbuffer;
	QString	askstring;
#define TEMPLATE_BUFFER_SIZE	128
	char templatename[TEMPLATE_BUFFER_SIZE];
	int rc;

// ****** TODO: make it do something sensible with permissions of the output file.	

		// Does this file already exist?	
	if(stat(filename,&statbuffer)<0) {
		errno = 0;	// Clear errno.
			// Nope, it doesn't seem to exist. We don't have to be quite as
			// careful when we write the file out.
		fhandle = fopen((const char *)filename,"w");
		if(fhandle==0) {
				// Try to work out what has happened here.
			errorstring = "An error occurred while opening '";
			errorstring += filename;
			errorstring += "' for writing. The operating system has this to report about the error: ";
			errorstring += strerror(errno);
			return WRITE_ERROR;
		}
			// errno
		rc = writeFirewall(fhandle,errorstring);
		if(rc!=OK) {
				// Something went wrong.
			fclose(fhandle);	// Close the file.
				
				// Did we leave a dead file?
			if(stat(filename,&statbuffer)==0) {
				// The dead file exists.
				errno = 0;
				unlink(filename);	// Delete the dead file.
			}
			return rc;
			
		}
		fclose(fhandle);
			
			// Make the written script executable.
		stat(filename,&statbuffer);
		chmod(filename, statbuffer.st_mode | S_IXUSR);

	} else {
			// We need to cook up a template for the tmp file.
			// We want the tmp file to be in the same dir as the file
			// it must replace. This is mainly so that we can avoid attempting
			// to rename a file across filesystems.
		strncpy(templatename,filename,TEMPLATE_BUFFER_SIZE);
		char *a;				// This just copies
		char *slash;			// guarddogXXXXXX over the filename
		slash = templatename-1;	// and keeps the same directory.
		a = templatename;
		while(*a!=0) {
			if(*a=='/') {
				slash = a;
			}
			a++;
		}
		strncpy(slash+1,"guarddogXXXXXX",TEMPLATE_BUFFER_SIZE-((templatename-slash)-1));
		
			// Now, open a file in that dir.
		tmpfd = mkstemp(templatename);
		if(tmpfd<0) {
			errorstring = "Error writing file. The operating system reports:\n";
			errorstring += strerror(errno);
			errno = 0;
			return WRITE_ERROR;
		}		

			// Lets wrap some STDIO around that fd.
		fhandle = fdopen(tmpfd,"w");
		if(fhandle==0) {
				// WTF has stuffed up now.
			errorstring = "Error writing file. The operating system reports:\n";
			errorstring += strerror(errno);
			errno = 0;
			close(tmpfd);
			return WRITE_ERROR;
		}
		rc = writeFirewall(fhandle,errorstring);
			// Time to write out firewall.
		if(rc!=OK) {
				// Something went wrong.
			fclose(fhandle);	// Close the file.
				
				// Did we leave a dead file?
			if(stat(templatename,&statbuffer)==0) {
				// The dead file exists.
				errno = 0;
				unlink(templatename);	// Delete the dead file.
			}
			return rc;
		}
		fclose(fhandle);

		if(rename(templatename,filename)!=0) {
				// Delete the old file.
			unlink(templatename);
			errorstring = "Error writing file. An error occurred while renaming \n"
				"the temporary file over the '";
			errorstring += filename;
			errorstring += "'. The operating system reported: \n";
			errorstring += strerror(errno);	
			return WRITE_ERROR;
		}
			// Use the permission bits of the old version of the file for
			// our new replacement file. Also set the u+x bit.
		chmod(filename, statbuffer.st_mode | S_IXUSR);
	}
		
	modified = false;
	setTitle(filename);
	return rc;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::deleteContents()
{
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////
	factoryDefaults();

}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::setClientProtocol(int index,bool state) {
	allowedClientProtocols[index] = state;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::setServerProtocol(int index,bool state) {
	allowedServerProtocols[index] = state;
}

///////////////////////////////////////////////////////////////////////////
bool GuardDogDoc::getClientProtocol(int index) {
	return allowedClientProtocols[index];
}

///////////////////////////////////////////////////////////////////////////
bool GuardDogDoc::getServerProtocol(int index) {
	return allowedServerProtocols[index];
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::slotBlock(int index) {
	replyAction = REPLY_ACTION(index);
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::slotSetLogPackets(bool on) {
	logPackets = on;
}

///////////////////////////////////////////////////////////////////////////
void  GuardDogDoc::slotSetAllowPing(bool on) {
	allowPing = on;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::slotSetLogPing(bool on) {
	logPing = on;
}

///////////////////////////////////////////////////////////////////////////
bool GuardDogDoc::getLogPackets() {
	return logPackets;
}

///////////////////////////////////////////////////////////////////////////
bool GuardDogDoc::getAllowPing() {
	return allowPing;
}

///////////////////////////////////////////////////////////////////////////
bool GuardDogDoc::getLogPing() {
	return logPing;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::setInterfaceName(const char *name) {
	strncpy(interfacename,name,INTERFACE_NAME_LENGTH);
	interfacename[INTERFACE_NAME_LENGTH-1] = 0;	// Make sure it is null terminated.
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::getInterfaceName(char *name) {
	strncpy(name,interfacename,INTERFACE_NAME_LENGTH);
}


///////////////////////////////////////////////////////////////////////////
int GuardDogDoc::getReplyAction() {
	return replyAction;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::setReplyAction(REPLY_ACTION action) {
	replyAction = action;
}

///////////////////////////////////////////////////////////////////////////
int GuardDogDoc::writeFirewall(FILE *fhandle,QString &errorstring) {
	int i;
	ProtocolInfo *pinfo;
	char *logoption;
	char *nullstring = "";
	char *logswitch = " -l ";
	
	char *replystring;
	char *denystring = " DENY ";
	char *rejectstring = " REJECT ";
	char *acceptstring = " ACCEPT ";
	
	if(fprintf(fhandle,"#!/bin/sh\n## [GuardDog]\n"
		"# This firewall script was generated with \"GuardDog\" by Simon Edwards\n"
		"#\n# PLEASE DO NOT EDIT!\n#\n"
		"echo -en 'Starting GuardDog firewall...\\r\\f'\n"
		"echo -n \"    Shutting down networking & clearing IP chains...\"\n"
		"# Shut down all traffic\n"
		"/sbin/ipchains -P forward DENY\n"
		"/sbin/ipchains -P input DENY\n"
		"/sbin/ipchains -P output DENY\n\n"
		"# Delete any existing chains\n"
		"/sbin/ipchains -F forward\n"
		"/sbin/ipchains -F input\n"
		"/sbin/ipchains -F output\n"
		"echo -en 'done\\r\\f'\n"
		"ANYWHERE=\"any/0\"\n"
		"EXTERNAL_IF=\"")<0) { goto writeerror; }
	
	if(fprintf(fhandle,interfacename)<0) { goto writeerror; }
	if(fprintf(fhandle,"\"\n"
		"LOOP_IF=\"lo\"\n"
		"CLASS_A=\"10.0.0.0/8\"\n"
		"CLASS_B=\"172.16.0.0/12\"\n"
		"CLASS_C=\"192.168.0.0/16\"\n"
		"MULTICAST=\"240.0.0.0/3\"\n"
		"BROADCAST_0=\"0.0.0.0\"\n"
		"BROADCAST_1=\"255.255.255.255\"\n"
		"echo -n \"    Creating IP firewall chains...\"\n")<0) { goto writeerror; }
	
	if(logPackets) {
		if(fprintf(fhandle,"## [Log Blocked Packets]\n")<0) { goto writeerror; }
		logoption = logswitch;
	} else {
		logoption = nullstring;
	}
	
	if(replyAction==REPLY_ACTION_DENY) {
		replystring = denystring;
		if(fprintf(fhandle,"## [Deny Packets]\n")<0) { goto writeerror; }
	} else {
		replystring = rejectstring;
		if(fprintf(fhandle,"## [Reject Packets]\n")<0) { goto writeerror; }
	}
	
	if(fprintf(fhandle,"# Turn on kernel IP spoof protection\n")<0) goto writeerror;
	if(fprintf(fhandle,"echo 1 > /proc/sys/net/ipv4/conf/all/rp_filter\n")<0) goto writeerror;
// This below doesn't seem to work on all systems.
//	if(fprintf(fhandle,"# Stop ICMP echos (ping) to broadcast addresses\n")<0) goto writeerror;
//	if(fprintf(fhandle,"echo 1 > /proc/sys/net/ipv4/conf/all/icmp_echo_ignore_broadcasts\n")<0) goto writeerror;

// I'll enable this when I work out exactly what it does. ;-)
//	if(fprintf(fhandle,"echo 1 > /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts\n")<0) goto writeerror;

/*	
	if(fprintf(fhandle,"# Refuse packets claiming to be to or from a Class A private network\n")<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -s $CLASS_A %s -j %s\n",logoption,replystring)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -d $CLASS_A %s -j %s\n",logoption,replystring)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -s $CLASS_A %s -j REJECT\n",logoption)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -d $CLASS_A %s -j REJECT\n\n",logoption)<0) goto writeerror;
	if(fprintf(fhandle,"# Refuse packets claiming to be to or from a Class B private network\n")<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -s $CLASS_B %s -j %s\n",logoption,replystring)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -d $CLASS_B %s -j %s\n",logoption,replystring)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -s $CLASS_B %s -j REJECT\n",logoption)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -d $CLASS_B %s -j REJECT\n\n",logoption)<0) goto writeerror;
	if(fprintf(fhandle,"# Refuse packets claiming to be to or from a Class C private network\n")<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -s $CLASS_C %s -j %s\n",logoption,replystring)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -d $CLASS_C %s -j %s\n",logoption,replystring)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -s $CLASS_C %s -j REJECT\n",logoption)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -d $CLASS_C %s -j REJECT\n\n",logoption)<0) goto writeerror;
	if(fprintf(fhandle,"# Refuse packets claiming to be to or from the loopback interface\n")<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -s $LOOPBACK %s -j %s\n",logoption,replystring)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input  -i $EXTERNAL_IF -d $LOOPBACK %s -j %s\n",logoption,replystring)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -s $LOOPBACK %s -j REJECT\n",logoption)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -d $LOOPBACK %s -j REJECT\n\n",logoption)<0) goto writeerror;
	if(fprintf(fhandle,"# Refuse broadcast address SOURCE packets\n")<0) goto writeerror;
*/
		// This is a full on hardcore firewall. Deny by default. Since we want the machine to be able
		// to talk to itself for things like automountd etc, we allow all local IP traffic.
	if(fprintf(fhandle,"# Allow local IP traffic\n/sbin/ipchains -A input -i lo -j ACCEPT\n")<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i lo -j ACCEPT\n")<0) goto writeerror;
		
		// We should always drop packets dealing with broadcast addresses.
		// We don't want to be tricked into DOS'ing an innocent IP or flooding
		// our network connection.
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -s $BROADCAST_1 %s -j DENY\n",logoption)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -d $BROADCAST_0 %s -j DENY\n\n",logoption)<0) goto writeerror;
	if(fprintf(fhandle,"# Refuse multicast/anycast/broadcast addresses (in.h) (NET-3-HOWTO)\n")<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -s $MULTICAST %s -j DENY\n\n",logoption)<0) goto writeerror;

	if(fprintf(fhandle,"## [Clients]\n")<0) goto writeerror;
		// Output the stuff for the client protocols
	for(i=0,pinfo=pdb->clientsList.first(); pinfo!=0; i++,pinfo=pdb->clientsList.next()) {
		if(allowedClientProtocols[i]) {
			if(fprintf(fhandle,"## [%s]\n",pinfo->name)<0) goto writeerror;
			
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -p tcp -s $ANYWHERE %s -d $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowInTCPSource,pinfo->allowInTCP)==false) goto writeerror;
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -p tcp ! -y -d $ANYWHERE %s -s $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowInTCPSource,pinfo->allowInTCP)==false) goto writeerror;
				
				// Write the rule for allowing the connection out. (i.e. a syn packet out)
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -p tcp -s $ANYWHERE %s -d $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowOutTCPSource,pinfo->allowOutTCP)==false) goto writeerror;
				// Also allow for non-syn packets to travel freely boths ways on the allowed connection.
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -p tcp ! -y -d $ANYWHERE %s -s $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowOutTCPSource,pinfo->allowOutTCP)==false) goto writeerror;
				
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -p udp -s $ANYWHERE %s -d $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowInUDPSource,pinfo->allowInUDP)==false) goto writeerror;
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -p udp -s $ANYWHERE %s -d $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowOutUDPSource,pinfo->allowOutUDP)==false) goto writeerror;
		}
	}
	
	if(fprintf(fhandle,"## [Servers]\n")<0) goto writeerror;
//	allowedServerProtocols
		// Output the stuff for the server protocols
	for(i=0,pinfo=pdb->serversList.first(); pinfo!=0; i++,pinfo=pdb->serversList.next()) {
		if(allowedServerProtocols[i]) {
			if(fprintf(fhandle,"## [%s]\n",pinfo->name)<0) goto writeerror;
			
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -p tcp -s $ANYWHERE %s -d $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowInTCPSource,pinfo->allowInTCP)==false) goto writeerror;
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -p tcp ! -y -d $ANYWHERE %s -s $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowInTCPSource,pinfo->allowInTCP)==false) goto writeerror;
			
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -p tcp -s $ANYWHERE %s -d $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowOutTCPSource,pinfo->allowOutTCP)==false) goto writeerror;
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -p tcp ! -y -d $ANYWHERE %s -s $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowOutTCPSource,pinfo->allowOutTCP)==false) goto writeerror;
			
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -p udp -s $ANYWHERE %s -d $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowInUDPSource,pinfo->allowInUDP)==false) goto writeerror;
			if(fprintExpandRule(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -p udp -s $ANYWHERE %s -d $ANYWHERE %s -j ACCEPT\n",
				pinfo->allowOutUDPSource,pinfo->allowOutUDP)==false) goto writeerror;
		}
	}
		
		// Catch all
	if(fprintf(fhandle,"# Handle packets that were not allowed by the rules above.\n")<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -p tcp %s -j %s\n",logoption,replystring)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -p tcp %s -j REJECT\n",logoption)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -p udp %s -j %s\n",logoption,replystring)<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -p udp %s -j REJECT\n",logoption)<0) goto writeerror;

		// ICMP and Ping handling
	if(allowPing) {
		if(fprintf(fhandle,"## [Allow Ping]\n")<0) goto writeerror;
		replystring = acceptstring;
	} else {
		replystring = denystring;
	}
	if(logPing) {
		if(fprintf(fhandle,"## [Log Ping]\n")<0) goto writeerror;
		logoption = logswitch;
	} else {
		logoption = nullstring;
	}
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -p icmp -s $ANYWHERE -d $ANYWHERE --icmp-type echo-request %s -j %s\n",
		logoption,replystring)<0) goto writeerror;

		// ICMP catch all (except PING stuff). This needs to be tightened up in the future when
		// I have better idea about how to handle all the other kinds of ICMP traffic.
	if(fprintf(fhandle,"/sbin/ipchains -A input -i $EXTERNAL_IF -p icmp -s $ANYWHERE -d $ANYWHERE -j ACCEPT\n")<0) goto writeerror;
	if(fprintf(fhandle,"/sbin/ipchains -A output -i $EXTERNAL_IF -p icmp -s $ANYWHERE -d $ANYWHERE -j ACCEPT\n")<0) goto writeerror;

	if(fprintf(fhandle,"\necho -ne 'done\\r\\f'\necho -ne '    GuardDog firewall done.\\r\\f'\n## [End]\n")<0) goto writeerror;
	return OK;
	
writeerror:
	errorstring = "An error occurred while writing. \nThe operating system reports: ";
	errorstring += strerror(errno);
	errno = 0;
	return WRITE_ERROR;
}

///////////////////////////////////////////////////////////////////////////
//
// /sbin/ipchains -A input -i $EXTERNAL_IF -p tcp -y -s $ANYWHERE -d $ANYWHERE 0:112 -l -j DENY
//
//
#define PORT_BUFFER_SIZE 16
bool GuardDogDoc::fprintExpandRule(FILE *fhandle,const char *format,
		struct ProtocolInfo::PortSpec *leftspec,
		struct ProtocolInfo::PortSpec *rightspec) {
		
	struct ProtocolInfo::PortSpec *left;
	char tempbuffer[PORT_BUFFER_SIZE];
	
	for(left = leftspec; left!=0; left=left->next) {
		if((left->portStart==0) && (left->portEnd==65535)) {
			tempbuffer[0] = 0;
			fprintExpandRightPorts(fhandle,format,tempbuffer,rightspec);
		} else {
			if(left->portStart==left->portEnd) {
				sprintf(tempbuffer,"%u",left->portStart);
			} else {
				sprintf(tempbuffer,"%u:%u",left->portStart,left->portEnd);
			}
			if(fprintExpandRightPorts(fhandle,format,tempbuffer,rightspec)==false) {
				return false;
			}
		}
	}
	return true;	
}

///////////////////////////////////////////////////////////////////////////
bool GuardDogDoc::fprintExpandRightPorts(FILE *fhandle,const char *format,const char *left,
		struct ProtocolInfo::PortSpec *spec) {
	
	char tempbuffer[PORT_BUFFER_SIZE];
	struct ProtocolInfo::PortSpec *p;
	
	if(spec) {
		p = spec;
		while(p) {
			if(p->portStart==p->portEnd) {
				sprintf(tempbuffer,"%u",p->portStart);
			} else {
				sprintf(tempbuffer,"%u:%u",p->portStart,p->portEnd);
			}
			if(fprintf(fhandle,format,left,tempbuffer)<0) {
				return false;
			}
			p = p->next;
		}
	} else {
		if(fprintf(fhandle,format,left)<0) {
			return false;
		}
	}
	return true;		
}

///////////////////////////////////////////////////////////////////////////
int GuardDogDoc::readFirewall(FILE *fhandle,QString &errorstring) {
#define INPUT_BUFFER_SIZE	512
	char inputbuffer[INPUT_BUFFER_SIZE];
	int line;
	int i,j;
	bool badread;
	ProtocolInfo *pinfo;
	line = 1;

	badread = false;
	
	// These defines are so that we can keep track of what we are doing.
	// They are also useful for
#define SECTION_PRELUDE0	0
#define SECTION_PRELUDE1	1
#define SECTION_INTERFACE		2
#define SECTION_HEAD	3
#define SECTION_CLIENTS	4
#define SECTION_SERVERS	5
#define SECTION_END		6

	int section = SECTION_PRELUDE0;
	StringParse sp;
			
		// Check for #!/bin/sh
	if(fgets(inputbuffer,INPUT_BUFFER_SIZE,fhandle)==0) {
		badread = true;
		goto readerror;
	}
	if(strcmp(inputbuffer,"#!/bin/sh\n")!=0) {
		goto readerror;
	}
	section = SECTION_PRELUDE1;
	line++;
	
		// Check for the opening tag.
	if(fgets(inputbuffer,INPUT_BUFFER_SIZE,fhandle)==0) {
		badread = true;
		goto readerror;
	}
	if(strcmp(inputbuffer,"## [GuardDog]\n")!=0) {
		goto readerror;
	}
	section = SECTION_INTERFACE;
	logPackets = false;

		// We need to know which interface is the Internet interface.
	while(section==SECTION_INTERFACE) {
		line++;
		if(fgets(inputbuffer,INPUT_BUFFER_SIZE,fhandle)==0) {
			badread = true;
			goto readerror;
		}
		
		sp.setStr(inputbuffer);
		if(sp.compareSubword("EXTERNAL_IF=")) {
			sp.skipChars(12);
			if(sp.getString(interfacename,INTERFACE_NAME_LENGTH)) {
				section = SECTION_HEAD;
				continue;
			}
		}
	}
		
		// All the stuff that comes before the client protocols.
	while(section==SECTION_HEAD) {
		line++;
		if(fgets(inputbuffer,INPUT_BUFFER_SIZE,fhandle)==0) {
			badread = true;
			goto readerror;
		}
		if(strcmp(inputbuffer,"## [Log Blocked Packets]\n")==0) {
			logPackets = true;
			continue;
		}
		if(strcmp(inputbuffer,"## [Deny Packets]\n")==0) {
			replyAction = REPLY_ACTION_DENY;
			continue;
		}
		if(strcmp(inputbuffer,"## [Reject Packets]\n")==0) {
			replyAction = REPLY_ACTION_REJECT;
			continue;
		}
		if(strcmp(inputbuffer,"## [Clients]\n")==0) {
			section = SECTION_CLIENTS;
			continue;
		}
			// Just because we don't know what this line is, isn't necessarily a bad thing.
			// just loop again.
	}

		// Clear the allowed list of client protocols.
	j = pdb->clientsList.count();
	for(i=0; i<j; i++) {
		allowedClientProtocols[i] = false;
	}
	
		// Read and handle the client protocols.
	while(section==SECTION_CLIENTS) {
		line++;
		if(fgets(inputbuffer,INPUT_BUFFER_SIZE,fhandle)==0) {
			badread = true;
			goto readerror;
		}

		if(strcmp(inputbuffer,"## [Servers]\n")==0) {
			section = SECTION_SERVERS;
			continue;
		}
			//
		sp.setStr(inputbuffer);
		if(sp.compareSubword("## [")) {
			sp.skipChars(4);
				// Could be a client protocol.
			j = pdb->clientsList.count();
			if(j!=0) {	// This should always be !0.

				for(i=0,pinfo = pdb->clientsList.first(); pinfo!=0; i++,pinfo = pdb->clientsList.next()) {
					if(sp.compareSubword(pinfo->name)) {
						sp.mark();
						sp.skipChars(strlen(pinfo->name));
						if(sp.compareSubword("]\n")) {
								// Got a good match.
							allowedClientProtocols[i] = true;
							break;
						} else {
							sp.rollback();
						}
					}
				}
				if(pinfo==0) {
					// Found some thing that we did not expect.
					goto readerror;
				}
				continue;
			}
		}
			// Dunno, just skip it.
	}

	allowPing = false;
	logPing = false;
		// Clear the allowed list of server protocols.
	j = pdb->serversList.count();
	for(i=0; i<j; i++) {
		allowedServerProtocols[i] = false;
	}
		
		// Read and handle the client protocols.
	while(section==SECTION_SERVERS) {
		line++;
		if(fgets(inputbuffer,INPUT_BUFFER_SIZE,fhandle)==0) {
			badread = true;
			goto readerror;
		}

		if(strcmp(inputbuffer,"## [Allow Ping]\n")==0) {
			allowPing = true;
			continue;
		}
			
		if(strcmp(inputbuffer,"## [Log Ping]\n")==0) {
			logPing = true;
			continue;
		}

		if(strcmp(inputbuffer,"## [End]\n")==0) {
			section = SECTION_END;
			continue;
		}
			
			//
		sp.setStr(inputbuffer);
		if(sp.compareSubword("## [")) {
			sp.skipChars(4);
				// Could be a server protocol.
			j = pdb->serversList.count();
			if(j!=0) {	// This should always be !0.

				for(i=0,pinfo = pdb->serversList.first(); pinfo!=0; i++,pinfo = pdb->serversList.next()) {
					if(sp.compareSubword(pinfo->name)) {
						sp.mark();
						sp.skipChars(strlen(pinfo->name));
						if(sp.compareSubword("]\n")) {
								// Got a good match.
							allowedServerProtocols[i] = true;
							break;
						} else {
							sp.rollback();
						}
					}
				}
				if(pinfo==0) {
					// Found some thing that we did not expect.
					goto readerror;
				}
				continue;
			}
		}
			// Dunno, just skip it.
	}

	return OK;

readerror:
	if(!badread || feof(fhandle)) {
		switch(section) {
			case SECTION_PRELUDE0:
				errorstring = "Error reading firewall file.\n"
					"Expected to find '#!/bin/sh'. Perhaps this is not a GuardDog firewall script.";
				break;
				
			case SECTION_PRELUDE1:
				errorstring = "Error reading firewall file.\n"
					"Expected to find '## [GuardDog]'. Perhaps this is not a GuardDog firewall script.";
				break;
				
			case SECTION_INTERFACE:
				errorstring = "Error reading firewall file. The end of file was reached.\n"
					"Expected to find 'EXTERNAL_IF='.";
				break;
				
			case SECTION_HEAD:
				errorstring = "Error reading firewall file. The end of file was reached.\n"
					"Expected to find '## [Clients]'.";
				break;
				
			case SECTION_CLIENTS:
				errorstring = "Error reading firewall file. The end of file was reached.\n"
					"Expected to find '## [Servers]'.";
				break;
				
			case SECTION_SERVERS:
				errorstring = "Error reading firewall file. The end of file was reached.\n"
					"Expected to find '## [End]'.";
				break;
				
			case SECTION_END:	// This should not happen.
				errorstring = "Something wrong has happened. This should never happen...";
				break;
		}
		return PARSE_ERROR;
	} else {
		// We are not at the EOF. Something else must be wrong.
		errorstring = "Error reading firewall file. The operating system has this to report:\n";
		errorstring += strerror(errno);
		return READ_ERROR;
	}
}

///////////////////////////////////////////////////////////////////////////
int GuardDogDoc::applyFirewall(QString &errorstring) {
	FILE *fhandle;
	int tmpfd;
	struct stat statbuffer;
	QString	askstring;
	QString filename;
	int rc;
		
#define TEMPLATE_BUFFER_SIZE	128
	char templatename[TEMPLATE_BUFFER_SIZE];
	strcpy(templatename,"/tmp/guarddogXXXXXX");

	tmpfd = mkstemp(templatename);
	if(tmpfd<0) {
		errorstring = "Error writing temporary file. The operating system reports:\n";
		errorstring += strerror(errno);
		errno = 0;
		return WRITE_ERROR;
	}		

		// Lets wrap some STDIO around that fd.
	fhandle = fdopen(tmpfd,"w");
	if(fhandle==0) {
			// WTF has stuffed up now.
		errorstring = "Error writing temporary file. The operating system reports:\n";
		errorstring += strerror(errno);
		errno = 0;
		close(tmpfd);
		return WRITE_ERROR;
	}
		// Time to write out firewall.
	rc = writeFirewall(fhandle,errorstring);
	if(rc!=OK) {
			// Something went wrong.
		fclose(fhandle);	// Close the file.
				
			// Did we leave a dead file?
		if(stat(templatename,&statbuffer)==0) {
				// The dead file exists.
			errno = 0;
			unlink(templatename);	// Delete the dead file.
		}
		return rc;
	}
	fclose(fhandle);

		// Make this thing executable.
	chmod(templatename,S_IXUSR | S_IRUSR | S_IWUSR);
	
	filename = templatename;
	rc = runFirewall(filename,errorstring);

		// Ok, time to clean up the mess.
		// Delete the tmp file.
	unlink(templatename);
	return rc;
}

///////////////////////////////////////////////////////////////////////////
//
// The filename must not contain spaces or shell chars.
//
int GuardDogDoc::runFirewall(const QString &filename,QString &errorstring) {
	KProcess *proc;
	proc = new KProcess();
	QString command;
	QString konsolebin;
	
	konsolebin = kapp->kde_bindir();
	konsolebin += "/konsole";
	
	command = filename;
	command += ";read -p \"Press return to continue\"";
	// From the command line this cunstruct looks something like:
	// /usr/bin/konsole -nowelcome -caption "GuardDog: Starting Firewall" -e /bin/bash -e "rc.firewall;read -p \"Press return to continue\""
	*proc << ((const char *)konsolebin) << "/konsole" << "-nowelcome" << "-caption" << "GuardDog: Starting Firewall";
	*proc << "-e" << "/bin/sh" << "-c" << (const char *)command;
	
	proc->start(KProcess::Block,KProcess::NoCommunication);
		// Block, but it shoud not take long to run.
	return OK;
}

///////////////////////////////////////////////////////////////////////////
// This attempts to open the given filename.
// If it is not present, it succeeds and sets the current document name to
// the given filename. If it is present, but not readable as Guarddog script,
// it silently fails and does *not* set the document filename.
//
int GuardDogDoc::openDefault(const QString &filename) {
	FILE *fhandle;
	QFileInfo fileInfo(filename);
	QString errorstring;
	int rc;

	fhandle = fopen((const char *)filename,"r");
	if(fhandle==0) {
		if(errno==ENOENT) {
			title = fileInfo.fileName();
			absFilePath = fileInfo.absFilePath();	
			filePathValid = true;
			setTitle(filename);
			factoryDefaults();
			slotUpdateAllViews(0);
			modified = false;
		}
		return READ_ERROR;
	}
	
	rc = readFirewall(fhandle,errorstring);
	
	fclose(fhandle);
	if(rc==OK) {
		title = fileInfo.fileName();
		absFilePath = fileInfo.absFilePath();	
		filePathValid = true;
		setTitle(filename);
	}
	slotUpdateAllViews(0);
	modified = false;
	return rc;
}

///////////////////////////////////////////////////////////////////////////
void GuardDogDoc::factoryDefaults() {
	int j,i,p,count;
	ProtocolInfo *pinfo;
	char *defaultclients[] = {
		"domain",
		"HTTP",
		"HTTPSSL",
		"PassiveFTP",
		"SMTP",
		"POP",
		"NNTP",
		0
		};
	
	strcpy(interfacename,"ppp0");
	replyAction = REPLY_ACTION_DENY;
	
	logPackets = true;
	allowPing = true;
	logPing = true;
		
		// Clear the client protocol list		
	count = pdb->clientsList.count();
	for(i=0; i<count; i++) {
		allowedClientProtocols[i] = false;
	}
	
		// Clear the server protocol list.
	count = pdb->serversList.count();
	for(i=0; i<count; i++) {
		allowedServerProtocols[i] = false;
	}

		// Set some sane default client protocols
	for(p=0; defaultclients[p]!=0; p++) {
		j = pdb->clientsList.count();
		if(j!=0) {	// This should always be !0.
			for(i=0,pinfo = pdb->clientsList.first(); pinfo!=0; i++,pinfo = pdb->clientsList.next()) {
				if(strcmp(pinfo->name,defaultclients[p])==0) {
					// Got a good match.
					allowedClientProtocols[i] = true;
					break;
				}
			}
		}
	}
}
