/***************************************************************************
                          guarddogdoc.h  -  description
                             -------------------
    begin                : Thu Feb 10 20:57:36 EST 2000
    copyright            : (C) 2000 by Simon Edwards
    email                : simon@simonzone.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GUARDDOGDOC_H
#define GUARDDOGDOC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>

#include "protocoldb.h"

// forward declaration of the GuardDog classes
class GuardDogView;

/**	GuardDogDoc provides a document object for a document-view model.
  *
  * The GuardDogDoc class provides a document object that can be used in conjunction with the classes GuardDogApp and GuardDogView
  * to create a document-view model for standard KDE applications based on KApplication and KTMainWindow. Thereby, the document object
  * is created by the GuardDogApp instance and contains the document structure with the according methods for manipulation of the document
  * data by GuardDogView objects. Also, GuardDogDoc contains the methods for serialization of the document data from and to files.
  *
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 0.4 code generation
  */
class GuardDogDoc : public QObject
{
  Q_OBJECT
  public:
    /** Constructor for the fileclass of the application */
    GuardDogDoc(QWidget *parent, const char *name=0);
    /** Destructor for the fileclass of the application */
    ~GuardDogDoc();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void addView(GuardDogView *view);
    /** removes a view from the list of currently connected views */
    void removeView(GuardDogView *view);
    /** sets the modified flag for the document after a modifying action on the view connected to the document.*/
    void setModified(bool _m=true){ modified=_m; };
    /** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
    bool isModified(){ return modified; };
    /** deletes the document's contents */
    void deleteContents();
    /** initializes the document generally */
    bool newDocument();
    /** closes the acutal document */
    void closeDocument();

	enum ReturnCode {OK,READ_ERROR,WRITE_ERROR,PARSE_ERROR};

    /** loads the document by filename and format and emits the updateViews() signal */
    int openDocument(const QString &filename, QString &errorstring);
    /** saves the document under filename and format.*/	
    int saveDocument(const QString &filename, QString &errorstring);
	/** Runs the firewall and updates the current ipchains rules. */
	int applyFirewall(QString &errorstring);

	int openDefault(const QString &filename);
		
    /** sets the path to the file connected with the document */
    void setAbsFilePath(const QString &filename);
    /** returns the pathname of the current document file*/
    const QString &getAbsFilePath() const;
    /** sets the filename of the document */
    void setTitle(const QString &_t);
    /** returns the title of the document */
    const QString &getTitle() const;
	
	/** Runs the given firewall file. */
	int runFirewall(const QString &filename,QString &errorstring);
    int writeFirewall(FILE *fhandle,QString &errorstring);
	int readFirewall(FILE *fhandle,QString &errorstring);

	void setClientProtocol(int index,bool state);
	void setServerProtocol(int index,bool state);

    bool getAllowPing();
    bool getLogPing();
	bool getLogPackets();
	void getInterfaceName(char *name);
	bool getClientProtocol(int index);
	bool getServerProtocol(int index);
	
	enum REPLY_ACTION {REPLY_ACTION_DENY,REPLY_ACTION_REJECT};

    int getReplyAction();
    void setReplyAction(REPLY_ACTION action);

    /** the list of the views currently connected to the document */
    static QList<GuardDogView> *pViewList;	
    void setInterfaceName(const char *name);
	bool isFilePathValid();

  public slots:
    /** calls repaint() on all views connected to the document object and is called by the view by which the document has been changed.
     * As this view normally repaints itself, it is excluded from the paintEvent.
     */
    void slotUpdateAllViews(GuardDogView *sender);

	void slotSetAllowPing(bool on);
	void slotSetLogPing(bool on);
	void slotSetLogPackets(bool on);
	void slotBlock(int index);

  private:
    /** the modified flag of the current document */
    bool modified;
    bool filePathValid;
    QString title;
    QString absFilePath;
	bool *allowedClientProtocols;
	bool *allowedServerProtocols;
	
	bool logPackets;
	bool allowPing;
	bool logPing;
#define INTERFACE_NAME_LENGTH	16	
	char interfacename[INTERFACE_NAME_LENGTH];
	REPLY_ACTION replyAction;
			
	bool fprintExpandRule(FILE *fhandle,const char *format,struct ProtocolInfo::PortSpec *leftspec,
		struct ProtocolInfo::PortSpec *rightspec);
	bool fprintExpandRightPorts(FILE *fhandle,const char *format,const char *left,
		struct ProtocolInfo::PortSpec *spec);
	
	void factoryDefaults();
		
	ProtocolDB *pdb;

};

#endif // GUARDDOGDOC_H
