/***************************************************************************
                          protocoldb.cpp  -  description
                             -------------------
    begin                : Thu Feb 10 2000
    copyright            : (C) 2000 by Simon Edwards
    email                : simon@simonzone.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "protocoldb.h"

///////////////////////////////////////////////////////////////////////////
ProtocolDB::ProtocolDB(){
    lastErrorString = 0;
    fhandle = 0;
    clientsList.setAutoDelete(true);
    serversList.setAutoDelete(true);
}

///////////////////////////////////////////////////////////////////////////

ProtocolDB::~ProtocolDB(){
    delete lastErrorString;
}

///////////////////////////////////////////////////////////////////////////
void ProtocolDB::cleanUpRead(int line,char *errorstring) {
    char temp[512];
    
    if(fhandle) {
        fclose(fhandle);
        fhandle = 0;
    }
    if(errorstring) {
        if(line!=0) {
            snprintf(temp,512,"Error Line: %d %s",line,errorstring);
            lastErrorString = new char[strlen(temp)+1];
            strcpy(lastErrorString,temp);
        } else {
            lastErrorString = new char[strlen(errorstring)+1];
            strcpy(lastErrorString,errorstring);
            
        }
    }
        
}


///////////////////////////////////////////////////////////////////////////

bool ProtocolDB::read(const char *filename) {
#define READBUFFER_SIZE 1024
    char linebuffer[READBUFFER_SIZE];
    StringParse parser;
    int line;
    ProtocolInfo *pi;
    pi = 0;
#define SECTION_NULL    0
#define SECTION_CLIENTS 1
#define SECTION_SERVERS 2    
    int section;
//fprintf(stderr,"In ProtocolDB read\n");
    section = SECTION_NULL; // We are not in a section yet.
    line = 1;
    
    fhandle = fopen(filename,"r");
    if(fhandle==0) {
            // The open failed, grab the error string.
        cleanUpRead(line,strerror(errno));
        return false;
    }    
    
    while(fgets(linebuffer,READBUFFER_SIZE,fhandle)) {
        parser.setStr(linebuffer);
        parser.skipWhite();
            // Is there any thing on this line?
        if(!parser.endOfString() && !parser.compareWord("#")) {
            if(parser.compareWord("[Clients]")) {
                section = SECTION_CLIENTS;
            } else {
                if(parser.compareWord("[Servers]")) {
                    section = SECTION_SERVERS;
                } else {
                
                    pi = new ProtocolInfo();
                    if(!pi->parseLine(parser)) {
                        delete pi;
                        cleanUpRead(line,"Parse error");
                        return false;
                    }
                    switch(section) {
                        case SECTION_NULL:
                        
                            break;
                            
                        case SECTION_CLIENTS:
                            clientsList.append(pi);
                            break;
                            
                        case SECTION_SERVERS:
                            serversList.append(pi);
                            break;
                    }
                }
            }
        }
        line++;
    }
    
        // Did we actually reach the end of file.
    if(!feof(fhandle)) {
        cleanUpRead(line,strerror(errno));
        return false;
    }
        
    cleanUpRead(0,0);
    return true;    
}

///////////////////////////////////////////////////////////////////////////

char *ProtocolDB::lastError() const {
    return lastErrorString;
}
