/***************************************************************************
                          protocolinfo.cpp  -  description
                             -------------------
    begin                : Thu Feb 10 2000
    copyright            : (C) 2000 by Simon Edwards
    email                : simon@simonzone.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "protocolinfo.h"
#include "stringparse.h"


///////////////////////////////////////////////////////////////////////////
ProtocolInfo::ProtocolInfo(){
    name = 0;
    description = 0;
    allowInTCP = 0;
    allowInTCPSource = 0;
    allowOutTCP = 0;
    allowOutTCPSource = 0;
    allowInUDP = 0;
    allowInUDPSource = 0;
    allowOutUDP = 0;
    allowOutUDPSource = 0;
}

///////////////////////////////////////////////////////////////////////////
ProtocolInfo::~ProtocolInfo(){
    destroyPortSpec(allowInTCP);
    destroyPortSpec(allowInTCPSource);
    destroyPortSpec(allowOutTCP);
    destroyPortSpec(allowOutTCPSource);
    destroyPortSpec(allowInUDP);
    destroyPortSpec(allowInUDPSource);
    destroyPortSpec(allowOutUDP);
    destroyPortSpec(allowOutUDPSource);
    
    delete name;
    delete description;
}

///////////////////////////////////////////////////////////////////////////
void ProtocolInfo::destroyPortSpec(struct PortSpec *x) {
	struct PortSpec *p,*q;
	
	p = x;
	while(p) {
		q = p->next;
		delete p;
		p = q;
	}
}

///////////////////////////////////////////////////////////////////////////
bool ProtocolInfo::parseLine(StringParse &parser) {
#define WORD_BUFFER_SIZE    512
    char wordbuffer[WORD_BUFFER_SIZE];

        // Read the name.
    if(!parser.getString(wordbuffer,WORD_BUFFER_SIZE)) {
            // Should do some kind of error string.
        return false;
    }
    name = new char[strlen(wordbuffer)+1];
    strcpy(name,wordbuffer);
    
    parser.skipWhite();
    
        // Read in the In TCP port list.
    if(!parsePortSpec(parser,&allowInTCP)) {
        return false;    
    }
    parser.skipWhite();
    
        // Read in the In TCP source port list.    
    if(!parsePortSpec(parser,&allowInTCPSource)) {
        return false;    
    }
    parser.skipWhite();
        
        // Read in the Out TCP port list.    
    if(!parsePortSpec(parser,&allowOutTCP)) {
        return false;    
    }
    parser.skipWhite();
        
        // Read in the Out TCP port list.    
    if(!parsePortSpec(parser,&allowOutTCPSource)) {
        return false;    
    }
    parser.skipWhite();

        // Read in the In UDP port list.
    if(!parsePortSpec(parser,&allowInUDP)) {
        return false;    
    }
    parser.skipWhite();
    
        // Read in the In UDP source port list.    
    if(!parsePortSpec(parser,&allowInUDPSource)) {
        return false;    
    }
    parser.skipWhite();
        
        // Read in the Out UDP port list.    
    if(!parsePortSpec(parser,&allowOutUDP)) {
        return false;    
    }
    parser.skipWhite();
        
        // Read in the Out UDP port list.    
    if(!parsePortSpec(parser,&allowOutUDPSource)) {
        return false;    
    }
    parser.skipWhite();
    
        
        // Read the description.
    if(!parser.getString(wordbuffer,WORD_BUFFER_SIZE)) {
            // Should do some kind of error string.
        return false;
    }
    description = new char[strlen(wordbuffer)+1];
    strcpy(description,wordbuffer);
    
    return true;
}

///////////////////////////////////////////////////////////////////////////
bool ProtocolInfo::parsePortSpec(StringParse &parser,struct PortSpec **x) {
#define TEMP_BUFFER_SIZE    256
    char tempbuffer[TEMP_BUFFER_SIZE];
    uint lport,rport;
    char c;
    StringParse sp;
    struct PortSpec *tempportspec;
    struct PortSpec **currentptr;
    
    currentptr = x;
            
    if(parser.getWord(tempbuffer,TEMP_BUFFER_SIZE)) {
        sp.setStr(tempbuffer);
        
            // Lets see if we have a port spec. '-' means no ports.
        sp.mark();
        if(sp.getChar(c)) {
            if(c=='-') {
                return true;
            }
			if(c=='*') {
                // Any port.
                tempportspec = new PortSpec();
                tempportspec->next = 0;
                tempportspec->portStart = 0;
                tempportspec->portEnd = 65535;
                *currentptr = tempportspec;
				return true;
			}
			if(c=='+') {
                // Only unprivliged ports.
                tempportspec = new PortSpec();
                tempportspec->next = 0;
                tempportspec->portStart = 1024;
                tempportspec->portEnd = 65535;
                *currentptr = tempportspec;
				return true;
			}

        } else {
                // We must be at the end of the string.
            return false;
        }
        sp.rollback();
    
        while(!sp.endOfString()) {
        
                // Encountered something wierd.
            if(!sp.getUint(lport)) {
            
                    // TODO: We should really look up any port names.
            
                destroyPortSpec(*x);
                *x = 0;
                return false;
            }
        
                // Is this the last part of the port spec.
                // It just contains a single port number.
            if(!sp.getChar(c)) {
                tempportspec = new PortSpec();
                tempportspec->next = 0;
                tempportspec->portStart = lport;
                tempportspec->portEnd = lport;
                *currentptr = tempportspec;
                return true;
            }
            
                // This is just a lone port number. But there is more waiting.
            if(c==',') {
                tempportspec = new PortSpec();
                tempportspec->next = 0;
                tempportspec->portStart = lport;
                tempportspec->portEnd = lport;
                *currentptr = tempportspec;
                currentptr = &((*currentptr)->next);
            } else {
                if(c==':') {    // A port range.
                    if(!sp.getUint(rport)) {
                        destroyPortSpec(*x);
		                *x = 0;
                        return false;
                    }
                        // This is a port range.
                    tempportspec = new PortSpec();
                    tempportspec->next = 0;
                    tempportspec->portStart = lport;
                    tempportspec->portEnd = rport;
                    *currentptr = tempportspec;
                    currentptr = &((*currentptr)->next);
                        // We have to check that the next char is a ',' indicating that
                        // there is another port spec. An end of string is cool too.
                    if(sp.getChar(c)) {
                        if(c!=',') {
                            destroyPortSpec(*x);
			                *x = 0;
                            return false;
                        }
                    }
                    
                } else {
                    destroyPortSpec(*x);
	                *x = 0;
                    return false;
                }
            }
        }
            // End of string hit.
        return true;
    } else {
        return false;
    }
    
}

///////////////////////////////////////////////////////////////////////////

